(******************************************************************************)
(*                             Start of crcmodel.h                            *)
(******************************************************************************)
(*                                                                            *)
(* Author : Ross Williams (ross@guest.adelaide.edu.au.).                      *)
(* Date   : 3 June 1993.                                                      *)
(* Status : Public domain.                                                    *)
(*                                                                            *)
(* Description : This is the header (.h) file for the reference               *)
(* implementation of the Rocksoft^tm Model CRC Algorithm. For more            *)
(* information on the Rocksoft^tm Model CRC Algorithm, see the document       *)
(* titled "A Painless Guide to CRC Error Detection Algorithms" by Ross        *)
(* Williams (ross@guest.adelaide.edu.au.). This document is likely to be in   *)
(* "ftp.adelaide.edu.au/pub/rocksoft".                                        *)
(*                                                                            *)
(* Note: Rocksoft is a trademark of Rocksoft Pty Ltd, Adelaide, Australia.    *)
(*                                                                            *)
(******************************************************************************)
(*                                                                            *)
(* How to Use This Package                                                    *)
(* -----------------------                                                    *)
(* Step 1: Declare a variable of type cm_t. Declare another variable          *)
(*         (p_cm say) of type p_cm_t and initialize it to point to the first  *)
(*         variable (e.g. p_cm_t p_cm = &cm_t).                               *)
(*                                                                            *)
(* Step 2: Assign values to the parameter fields of the structure.            *)
(*         If you don't know what to assign, see the document cited earlier.  *)
(*         For example:                                                       *)
(*            p_cm->cm_width = 16;                                            *)
(*            p_cm->cm_poly  = 0x8005L;                                       *)
(*            p_cm->cm_init  = 0L;                                            *)
(*            p_cm->cm_refin = TRUE;                                          *)
(*            p_cm->cm_refot = TRUE;                                          *)
(*            p_cm->cm_xorot = 0L;                                            *)
(*         Note: Poly is specified without its top bit (18005 becomes 8005).  *)
(*         Note: Width is one bit less than the raw poly width.               *)
(*                                                                            *)
(* Step 3: Initialize the instance with a call cm_ini(p_cm);                  *)
(*                                                                            *)
(* Step 4: Process zero or more message bytes by placing zero or more         *)
(*         successive calls to cm_nxt. Example: cm_nxt(p_cm,ch);              *)
(*                                                                            *)
(* Step 5: Extract the CRC value at any time by calling crc = cm_crc(p_cm);   *)
(*         If the CRC is a 16-bit value, it will be in the bottom 16 bits.    *)
(*                                                                            *)
(******************************************************************************)
(*                                                                            *)
(* Design Notes                                                               *)
(* ------------                                                               *)
(* PORTABILITY: This package has been coded very conservatively so that       *)
(* it will run on as many machines as possible. For example, all external     *)
(* identifiers have been restricted to 6 characters and all internal ones to  *)
(* 8 characters. The prefix cm (for Crc Model) is used as an attempt to avoid *)
(* namespace collisions. This package is endian independent.                  *)
(*                                                                            *)
(* EFFICIENCY: This package (and its interface) is not designed for           *)
(* speed. The purpose of this package is to act as a well-defined reference   *)
(* model for the specification of CRC algorithms. If you want speed, cook up  *)
(* a specific table-driven implementation as described in the document cited  *)
(* above. This package is designed for validation only; if you have found or  *)
(* implemented a CRC algorithm and wish to describe it as a set of parameters *)
(* to the Rocksoft^tm Model CRC Algorithm, your CRC algorithm implementation  *)
(* should behave identically to this package under those parameters.          *)
(*                                                                            *)
(******************************************************************************)

{$A-,B-,D+,E-,F-,I-,L+,N-,O-,R-,S-,V-} {$IfDef Ver60} {$G-,X-} {$EndIf}
{$IfDef Ver70} {$G-,P-,Q-,T-,X-,Y+}  {$EndIf}
{$IfDef Win32} {$H-} {$EndIf}

{        *********************************************************       }
{        *                    CRCModel.PAS 1.0                   *       }
{        *            Desined by Vladimir V. Vasil'ev            *       }
{        *                (""  PASCAL)                *       }
{        *********************************************************       }

{$I BBB_def.INC}

Unit CRCMODEL4Delphi;

Interface

uses BBB_Type;

(******************************************************************************)

(* The following definitions are extracted from my style header file which    *)
(* would be cumbersome to distribute with this package. The DONE_STYLE is the *)
(* idempotence symbol used in my style header file.                           *)

{type  p_ubyte_ = ^byte;}
type  p_ubyte_ = tPtrByteArr1;

type
{$IfNDef Win32}        { 18.2.2001 }
     int = integer;
{$Else}
     int = word;
{$EndIf}
//     ulong = longint;
     ulong = longword;
     bool  = boolean;
(******************************************************************************)

(* CRC Model Abstract Type *)
(* ----------------------- *)
(* The following type stores the context of an executing instance of the  *)
(* model algorithm. Most of the fields are model parameters which must be *)
(* set before the first initializing call to cm_ini.                      *)
type  cm_t = record
   cm_width : int;   (* Parameter: Width in bits [8,32].       *)
   cm_poly  : ulong;    (* Parameter: The algorithm's polynomial. *)
   cm_init  : ulong;    (* Parameter: Initial register value.     *)
   cm_refin : bool;   (* Parameter: Reflect input bytes?        *)
   cm_refot : bool;   (* Parameter: Reflect output CRC?         *)
   cm_xorot : ulong;   (* Parameter: XOR this to output CRC.     *)

   cm_reg : ulong;     (* Context: Context during execution.     *)
  end;  { cm_t }
type  p_cm_t = ^cm_t;

type tCRCTable  = cm_t;            (*  㤮⢠  *)
type tPtrCRCTable  = p_cm_t;       (*  㤮⢠  *)


 { 祭, ᯮ㥬   CRC 娢஬ ZIP }
const rZIP32CRCTable : tCRCTable = (
      cm_width : 32;         (* Parameter: Width in bits [8,32].       *)
      cm_poly  : $04C11DB7;  (* Parameter: The algorithm's polynomial. *)
      cm_init  : $FFFFFFFF;  (* Parameter: Initial register value.     *)
      cm_refin : TRUE;       (* Parameter: Reflect input bytes?        *)
      cm_refot : TRUE;       (* Parameter: Reflect output CRC?         *)
      cm_xorot : $FFFFFFFF   (* Parameter: XOR this to output CRC.     *)
      );                     (* Context: Context during execution.     *)

 { 祭, ᯮ㥬   CRC ⥬ "-8" }
const  rDomino8CRCTable : tCRCTable = (
      cm_width : 32;         (* Parameter: Width in bits [8,32].       *)
      cm_poly  : $04C11DB7;  (* Parameter: The algorithm's polynomial. *)
      cm_init  : $28041967;  (* Parameter: Initial register value.     *)
      cm_refin : TRUE;       (* Parameter: Reflect input bytes?        *)
      cm_refot : TRUE;       (* Parameter: Reflect output CRC?         *)
      cm_xorot : $18915031   (* Parameter: XOR this to output CRC.     *)
         { Check  : $F2D6D3B3 }
      );                     (* Context: Context during execution.     *)

(******************************************************************************)

(* Functions That Implement The Model *)
(* ---------------------------------- *)
(* The following functions animate the cm_t abstraction. *)

procedure cm_ini (p_cm : p_cm_t);
(* Initializes the argument CRC model instance.          *)
(* All parameter fields must be set before calling this. *)

procedure cm_nxt (p_cm : p_cm_t; ch : byte);
(* Processes a single message byte [0,255]. *)

procedure cm_blk (p_cm : p_cm_t;  blk_adr : p_ubyte_; blk_len : ulong);
(* Processes a block of message bytes. *)

function cm_crc (p_cm : p_cm_t) : ulong;
(* Returns the CRC value for the message bytes processed so far. *)

(******************************************************************************)

(* Functions For Table Calculation *)
(* ------------------------------- *)
(* The following function can be used to calculate a CRC lookup table.        *)
(* It can also be used at run-time to create or check static tables.          *)

function cm_tab (p_cm : p_cm_t; index : int) : ulong;
(* Returns the i'th entry for the lookup table for the specified algorithm.   *)
(* The function examines the fields cm_width, cm_poly, cm_refin, and the      *)
(* argument table index in the range [0,255] and returns the table entry in   *)
(* the bottom cm_width bytes of the return value.                             *)

(******************************************************************************)


(******************************************************************************)
(*                             End of crcmodel.h                              *)
(******************************************************************************)

(**************************************************************)
(*           "㭪樨"  CRC                       *)
(**************************************************************)

function FileZipCRC_ByHandle (var F : File; var iiCRC : ulong) : integer;
function FileZipCRC_ByName (stFileName : string; var iiCRC : ulong) : integer;
 { 㭪樨 뢠 CRC 㪠 䠩  ᭮ ࠬ஢, }
 { ᯮ㥬 娢஬ ZIP. }
 {  㭪樨:  襭, ᤨ  ଠ쭮,  - errOK }
 {   errOK ⠭ CRC 頥  iiCRC }

Implementation

(******************************************************************************)
(*                             Start of crcmodel.c                            *)
(******************************************************************************)
(*                                                                            *)
(* Author : Ross Williams (ross@guest.adelaide.edu.au.).                      *)
(* Date   : 3 June 1993.                                                      *)
(* Status : Public domain.                                                    *)
(*                                                                            *)
(* Description : This is the implementation (.c) file for the reference       *)
(* implementation of the Rocksoft^tm Model CRC Algorithm. For more            *)
(* information on the Rocksoft^tm Model CRC Algorithm, see the document       *)
(* titled "A Painless Guide to CRC Error Detection Algorithms" by Ross        *)
(* Williams (ross@guest.adelaide.edu.au.). This document is likely to be in   *)
(* "ftp.adelaide.edu.au/pub/rocksoft".                                        *)
(*                                                                            *)
(* Note: Rocksoft is a trademark of Rocksoft Pty Ltd, Adelaide, Australia.    *)
(*                                                                            *)
(******************************************************************************)
(*                                                                            *)
(* Implementation Notes                                                       *)
(* --------------------                                                       *)
(* To avoid inconsistencies, the specification of each function is not echoed *)
(* here. See the header file for a description of these functions.            *)
(* This package is light on checking because I want to keep it short and      *)
(* simple and portable (i.e. it would be too messy to distribute my entire    *)
(* C culture (e.g. assertions package) with this package.                     *)
(*                                                                            *)
(******************************************************************************)

(******************************************************************************)

(* The following definitions make the code more readable. *)

{
function BITMASK (X : byte) : ulong;
begin
 BITMASK := longint(1) shl X;
end;
}
(*
function BITMASK (X : byte) : ulong; assembler;
asm
  Mov  Cl, X
  Mov  Ch, Cl   { save X }
  And  Cl, $07  { make (X mod 7) }
  Mov  Bl, 1
  Shl  Bl, Cl
  Xor  Ax, Ax
  Xor  Dx, Dx
  And  Ch, $F8  { make a la (X div 7) }
  Or   Ch, Ch
  JnZ  @loc_Check_1
  Mov  Al, Bl
  Jmp  @loc_Exit
@loc_Check_1 :
  Cmp  Ch, $08
  JnZ  @loc_Check_2
  Mov  Ah, Bl
  Jmp  @loc_Exit
@loc_Check_2 :
  Cmp  Ch, $10
  JnZ  @loc_Check_3
  Mov  Dl, Bl
  Jmp  @loc_Exit
@loc_Check_3:
  Mov  Dh, Bl
@loc_Exit :
end;  { BITMASK }
*)

const auiBITMASK : array [0..31] of ulong =
         ($00000001, $00000002, $00000004, $00000008,
             $00000010, $00000020, $00000040, $00000080,
          $00000100, $00000200, $00000400, $00000800,
             $00001000, $00002000, $00004000, $00008000,
          $00010000, $00020000, $00040000, $00080000,
             $00100000, $00200000, $00400000, $00800000,
          $01000000, $02000000, $04000000, $08000000,
             $10000000, $20000000, $40000000, $80000000);
{
function BITMASK (X : byte) : ulong;
begin
 BITMASK := auiBITMASK [X];
end;
}
const MASK32 = $FFFFFFFF;

(******************************************************************************)

function reflect_0 (v : ulong; b : int) : ulong;
 { Returns the value v with the bottom b [0,32] bits reflected. }
 { Example: reflect(0x3e23L,3) == 0x3e26                        }
var i : int;
    t : ulong;
begin
 t := v;
 for i:=0 to (b-1) do
   begin
     if ((t AND 1) <> 0) then
       v := v OR auiBITMASK [(b-1)-i]
     else
       v := v AND ($FFFFFFFF XOR auiBITMASK [(b-1)-i]);
     t := t shr 1;
   end;
 reflect_0 := v;
end;

const auiTOP_FF_MASK : array [0..32] of ulong =
         ($FFFFFFFF, $FFFFFFFE, $FFFFFFFC, $FFFFFFF8,
             $FFFFFFF0, $FFFFFFE0, $FFFFFFC0, $FFFFFF80,
          $FFFFFF00, $FFFFFE00, $FFFFFC00, $FFFFF800,
             $FFFFF000, $FFFFE000, $FFFFC000, $FFFF8000,
          $FFFF0000, $FFFE0000, $FFFC0000, $FFF80000,
             $FFF00000, $FFE00000, $FFC00000, $FF800000,
          $FF000000, $FE000000, $FC000000, $F8000000,
             $F0000000, $E0000000, $C0000000, $80000000,
          $00000000);

function reflect (ulData : ulong; iBits : int) : ulong;
 { Returns the value v with the bottom b [0,32] bits reflected. }
 { Example: reflect(0x3e23L,3) == 0x3e26                        }
var  ulTop,
     ulLow,
     ulLow_New : ulong;
     iLoop : longint;
begin
 //      
  ulTop := ulData AND auiTOP_FF_MASK [iBits];

 //  
  ulLow := ulData AND ($FFFFFFFF XOR auiTOP_FF_MASK [iBits]);
 // ""  iBits .
  ulLow_New := 0;
  for iLoop:= 1 to iBits do begin
    ulLow_New := ulLow_New OR
       ((ulLow shr (iLoop - 1)) AND $1) shl (iBits - iLoop);
  end;

  result := ulTop OR ulLow_New;

end;  //  reflect


(* * )
function reflect (ulData : ulong; iBits : int) : ulong;
 { Returns the value v with the bottom b [0,32] bits reflected. }
 { Example: reflect(0x3e23L,3) == 0x3e26                        }
begin
 asm
  {  iBits <= 16  iBits > 16: ࠧ  }
   Cmp iBits, 16
   Ja @loc_More16Bits

  { ⠥ ⮫쪮  訬 ᫮ ulData }
   Mov  Cx, iBits
   Mov  Ax, word ptr ulData   ;{ "筨"  }
   Mov  Bx, $FFFF             ;{ ନ㥬  }
   Shl  Bx, Cl
   Xor  Dx, Dx                ;{ ਥ  }
@loc_1 :
   Shr  Ax, 1      { 뤢 ,  頥  CF }
   Rcl  Dx, 1      { . ᤢ Dx "१ ७": }
                   { 祭 CF   訩  Dx }
   Loop @loc_1

  { ନ㥬 "ࠦ" 襥 ᫮ }
   And  word ptr ulData, Bx
   Or   word ptr ulData, Dx
   Jmp  @loc_EndAsm

@loc_More16Bits :
  { 砫, 筮  <=16 "ࠦ" 襥 ᫮ }
   Mov  Cx, 16
   Mov  Ax, word ptr ulData   ;{ "筨"  }
   Xor  Dx, Dx                ;{ ਥ  }
@loc_2 :
   Shr  Ax, 1      { 뤢 ,  頥  CF }
   Rcl  Dx, 1      { . ᤢ Dx "१ ७": }
                   { 祭 CF   訩  Dx }
   Loop @loc_2

  {    㦭 ""  訬 ᫮ }
  { :  Dx  "" ! }
   Mov  Cx,  iBits
   Sub  Cx,  16
   Mov  Ax, word ptr ulData[2]   ;{ "筨"  }
   Mov  Bx, $FFFF             ;{ ନ㥬  }
   Shl  Bx, Cl
   Xor  Di, Di                   ;{ ਥ  }
@loc_3 :
   Shr  Ax, 1      { 뤢 ,  頥  CF }
   Rcl  Dx, 1      { . ᤢ Di "१ ७": }
                   { 祭 CF   訩  Dx, }
                   { 騩  Dx 頥  CF }
   Rcl  Di, 1      { . ᤢ Di "१ ७": }
                   { 祭 CF   訩  Di }
   Loop @loc_3

  {  "" Di+Dx  "ࠦ"  }
  { ନ㥬 "ࠦ" 襥 ᫮ }
   And  word ptr ulData[2], Bx
   Or   word ptr ulData[2], Di
  { 㥬 ନ஢ 襥 ᫮ }
   Mov  word ptr ulData,    Dx

@loc_EndAsm :
 end; { asm }

 reflect := ulData;
end;
(* *)


(******************************************************************************)

const _widmask : array [0..32] of ulong =
         ($00000000, $00000001, $00000003, $00000007,
             $0000000F, $0000001F, $0000003F, $0000007F,
          $000000FF, $000001FF, $000003FF, $000007FF,
             $00000FFF, $00001FFF, $00003FFF, $00007FFF,
          $0000FFFF, $0001FFFF, $0003FFFF, $0007FFFF,
             $000FFFFF, $001FFFFF, $003FFFFF, $007FFFFF,
          $00FFFFFF, $01FFFFFF, $03FFFFFF, $07FFFFFF,
             $0FFFFFFF, $1FFFFFFF, $3FFFFFFF, $7FFFFFFF,
          $FFFFFFFF);

(*
function widmask (p_cm : p_cm_t) : ulong;
 { Returns a longword whose value is (2^p_cm->cm_width)-1.     }
 { The trick is to do this portably (e.g. without doing <<32). }
var _1L : ulong;
begin
  _1L := 1;
  widmask := ((( _1L shl (p_cm^.cm_width-1))-_1L) shl 1) OR _1L;
end;
*)

(******************************************************************************)

procedure cm_ini (p_cm : p_cm_t);
begin
 p_cm^.cm_reg := p_cm^.cm_init;
end;

(******************************************************************************)

{---------------------}

function ULongShl1 (ulData : ulong) : ulong;
begin
  Result := ulData shl 1;
end;

(* * )
function ULongShl1 (ulData : ulong) : ulong; near; assembler;
 { This function made and was begun to use 22.08.2000 }
 { Perfomance increaced up to 1.5 times /from ~13.50 sec to ~8.40 sec/ }
asm
{
  Pop Ax
  Pop Dx
}
  Mov Ax, word ptr ulData+0
  Mov Dx, word ptr ulData+2   { Ax+Dx = ulData now}
  Shl Ax, 1
  Rcl Dx, 1
end;  { ULongShl1 }
(* *)

(*
function ULongShl1 (ulData : ulong) : ulong;
 { This function made and was begun to use 22.08.2000 }
 { Change ASM to inline decreaced time from ~8.40 to ~7.91 sec/ }
 inline(
   $58/              {  Pop Ax }
   $5A/              {  Pop Dx }
   $D1/$E0/          {  Shl Ax,1 }
   $D1/$D2);         {  Rcl Dx,1 }
*)

{---------------------}

procedure cm_nxt (p_cm : p_cm_t; ch : byte);
var i : int;
    uch : ulong;
    topbit : ulong;
         iWidth : int;
         ulReg, ulPoly  : ulong;
begin
 uch := ch;

        iWidth := p_cm^.cm_width;
        ulPoly := p_cm^.cm_poly;

{ topbit := auiBITMASK [p_cm^.cm_width-1]; }
 topbit := auiBITMASK [iWidth-1];
 if (p_cm^.cm_refin) then
   uch := reflect (uch,8);
{ p_cm^.cm_reg :=  p_cm^.cm_reg XOR (uch shl (p_cm^.cm_width-8)); }
 ulReg :=  p_cm^.cm_reg XOR (uch shl (iWidth-8));
 for i:=0 to (8-1) do
   begin
{
    if ((p_cm^.cm_reg AND topbit) <> 0) then
       p_cm^.cm_reg := (p_cm^.cm_reg shl 1) XOR p_cm^.cm_poly
    else
       p_cm^.cm_reg :=  p_cm^.cm_reg shl 1;
    p_cm^.cm_reg := p_cm^.cm_reg AND widmask (p_cm);
}
    if ((ulReg AND topbit) <> 0) then
{       ulReg := (ulReg shl 1) XOR ulPoly}
       ulReg := ULongShl1 (ulReg) XOR ulPoly
    else
{       ulReg :=  ulReg shl 1;}
      ulReg :=  ULongShl1 (ulReg);
    ulReg := ulReg AND _widmask [iWidth];
   end;
  p_cm^.cm_reg := ulReg;
end;

(******************************************************************************)

procedure cm_blk (p_cm : p_cm_t; blk_adr : p_ubyte_; blk_len : ulong);
var  wLoop : word;
begin
  for wLoop := 1 to blk_len do
    begin
      cm_nxt (p_cm, blk_adr^[wLoop]);
    end;
end;

(******************************************************************************)

function cm_crc (p_cm : p_cm_t) : ulong;
begin
 if (p_cm^.cm_refot) then
    cm_crc := p_cm^.cm_xorot XOR reflect(p_cm^.cm_reg, p_cm^.cm_width)
 else
    cm_crc := p_cm^.cm_xorot XOR p_cm^.cm_reg;
end;

(******************************************************************************)

function cm_tab (p_cm : p_cm_t; index : int) : ulong;
var i : int;
    r : ulong;
    topbit : ulong ;
    inbyte : ulong;
begin
 topbit := auiBITMASK[p_cm^.cm_width-1];
 inbyte := index;

 if (p_cm^.cm_refin) then
   inbyte := reflect(inbyte,8);
 r := inbyte shl (p_cm^.cm_width-8);
 for i:=0 to (8-1) do
   if ((r AND topbit) <> 0) then
{      r := (r shl 1) XOR p_cm^.cm_poly}
      r := ULongShl1 (r) XOR p_cm^.cm_poly
   else
{      r := r shl 1;}
      r := ULongShl1 (r);

 if (p_cm^.cm_refin) then
   r := reflect(r, p_cm^.cm_width);
{ cm_tab := r AND widmask(p_cm); }
 cm_tab := r AND _widmask [p_cm^.cm_width];
end;

(******************************************************************************)
(*                             End of crcmodel.c                              *)
(******************************************************************************)

(**************************************************************)
(*           "㭪樨"  CRC                       *)
(**************************************************************)

 { 祭, ᯮ㥬   CRC 娢஬ ZIP }
var rCRCTable4FileZipCRC : tCRCTable;
    abBuff4FileZipCRC : array [0..$1000] of byte;

 { 㭪樨 뢠 CRC 㪠 䠩  ᭮ ࠬ஢, }
 { ᯮ㥬 娢஬ ZIP. }
 {  㭪樨:  襭, ᤨ  ଠ쭮,  - errOK }
 {   errOK ⠭ CRC 頥  iiCRC }
function FileZipCRC_ByHandle (var F : File; var iiCRC : ulong) : integer;
label loc_IOErrorExit;
var iSaveFilePos : longint;
    wRead : integer;
    _InOutRes : integer;
begin
  iSaveFilePos := FilePos (F);

  Seek (F, 0);
  _InOutRes := IOResult ();
  if (_InOutRes <> 0) then
    begin
loc_IOErrorExit :
      FileZipCRC_ByHandle := _InOutRes;
      exit;
    end;

  cm_ini (@rCRCTable4FileZipCRC);  { 樠㥬  }
  while (NOT EOF (F)) do
    begin
      BlockRead (F, abBuff4FileZipCRC, SizeOf (abBuff4FileZipCRC), wRead);
      _InOutRes := IOResult ();
      if (_InOutRes <> 0) then
        goto loc_IOErrorExit;

     {  । 樨  }
      cm_blk (@rCRCTable4FileZipCRC, @abBuff4FileZipCRC, wRead);

    end;  { while (NOT EOF (F)) do }

 { "蠥"  }
  iiCRC := cm_crc (@rCRCTable4FileZipCRC);

 { ⠭   䠩 }
  Seek (F, iSaveFilePos);
  _InOutRes := IOResult ();
  if (_InOutRes <> 0) then
    goto loc_IOErrorExit;

 {  ஢ ᯥ譮 }
  FileZipCRC_ByHandle := 0;  { errOK }

end;  { FileZipCRC_ByHandle }

 {--------------------------------------------------------}

function FileZipCRC_ByName (stFileName : string; var iiCRC : ulong) : integer;
var F : File;
    wSaveFileMode : word;
    iErr : integer;
    _InOutRes : integer;
begin
  wSaveFileMode := FileMode;
  FileMode := 0; { ReadOnly mode }

  assign (F, stFileName);
  Reset (F, 1);
  FileMode := wSaveFileMode;
  _InOutRes := IOResult ();
  if (_InOutRes <> 0) then
    begin
      FileZipCRC_ByName := _InOutRes;
      exit;
    end;

 { 뢠 CRC }
  iErr := FileZipCRC_ByHandle (F, iiCRC);

 { 뢢 䠩 }
  close (F);
  if (IOResult <> 0) then;

 { 頥   }
  FileZipCRC_ByName := iErr;

end;  { FileZipCRC_ByName }

begin
  rCRCTable4FileZipCRC := rZIP32CRCTable;
end.
